#!/bin/bash

usage_and_exit () {
	cat >&2 << EOF
Usage: $0 TYPE TARGET JOB CWD

CD packaging and deploying script wrapper.

Arguments:

  TYPE       Either 'snapshot' or 'stable'.
  TARGET     Target distro. Determines script to be called.
  JOB        Either 'package' or 'deploy'
  CWD        Working directory.

EOF
	exit 1
}

usage_and_exit_with_msg () {
	echo Error: "$1" >&2
	echo "" >&2
	usage_and_exit
}

exit_with_msg () {
	echo Error: "$1" >&2
	exit $2
}

tolower () {
	echo "$1" | tr '[:upper:]' '[:lower:]'
}

function_exists () {
	type "$1" >/dev/null 2>/dev/null
}

# ---------------------------------------------------------------------------------------------------------------------

gpg_import () {
	if [[ -x /usr/libexec/gpg-preset-passphrase ]]; then
		GPGPP=/usr/libexec/gpg-preset-passphrase
	elif [[ -x /usr/lib/gnupg/gpg-preset-passphrase ]]; then
		GPGPP=/usr/lib/gnupg/gpg-preset-passphrase
	else
		echo "gpg-preset-passphrase not found"
		exit 1
	fi

	# initialize agent
	mkdir -p ~/.gnupg
	echo -ne 'allow-preset-passphrase\nmax-cache-ttl 60480000\ndefault-cache-ttl 60480000\n' > ~/.gnupg/gpg-agent.conf
	gpg-connect-agent reloadagent /bye

	# import key
	echo "$GPG_PRIVATE_KEY" | gpg --no-tty --batch --import -
	KEYGRIP=$(gpg --with-keygrip -k $GPG_KEYID | grep Keygrip | cut -d= -f2 | tr -d ' ')
	$GPGPP --preset --passphrase "$GPG_PASSPHRASE" "$KEYGRIP"
}

# ---------------------------------------------------------------------------------------------------------------------

SCRIPT_ROOT=$(realpath $(dirname $0))
GIT_ROOT=$(realpath "$SCRIPT_ROOT"/../../../../)
TYPE=
TARGET=
CWD=

# ---------------------------------------------------------------------------------------------------------------------
# check input
[[ $# -ne 4 ]] && usage_and_exit

TYPE=$(tolower "$1")
[[ $TYPE != "snapshot" && $TYPE != "stable" ]] && usage_and_exit_with_msg "Invalid type."

TARGET=$(tolower "$2")
DISTROSCRIPT="$SCRIPT_ROOT/package-$TARGET.sh"
[[ ! -e "$DISTROSCRIPT" ]] && usage_and_exit_with_msg "Invalid distro. File $(basename $DISTROSCRIPT) not found."

JOB=$(tolower "$3")
[[ $JOB != "package" && $JOB != "deploy" ]] && usage_and_exit_with_msg "Invalid job."

CWD=$(realpath "$4")
[[ ! -d "$CWD" ]] && usage_and_exit_with_msg "Directory $4 does not exist".

# ---------------------------------------------------------------------------------------------------------------------
# source distroscript

echo "Sourcing '$DISTROSCRIPT'"
source "$DISTROSCRIPT"

function_exists "package" || exit_with_msg "Function package() not found in package-$TARGET.sh" 2
function_exists "before"  || exit_with_msg "Function before() not found in package-$TARGET.sh" 2
function_exists "deploy"  || exit_with_msg "Function deploy() not found in package-$TARGET.sh" 2

# ---------------------------------------------------------------------------------------------------------------------
# run before...

# ---------------------------------------------------------------------------------------------------------------------
# create archive inside $CWD

if [[ $JOB == "package" ]]; then
	before
	cd $GIT_ROOT

	if [[ $TYPE == "snapshot" ]]; then
		VERSION="$(git describe --long --tags --match="v*" |  sed 's/^v//;s/\([^-]*-g\)/r\1/;s/-/./g')"
	elif [[ $TYPE == "stable" ]]; then
		VERSION="$(git describe --tags --match="v*" --abbrev=0 | sed 's/^v//g')"
	fi

	ARCHIVE_DIR="algorithms-library_$VERSION"
	ARCHIVE="$ARCHIVE_DIR.tar.gz"
	echo "Creating archive $ARCHIVE"

	if [[ $TYPE == "snapshot" ]]; then
		git archive --format=tar.gz --prefix="$ARCHIVE_DIR/" HEAD -o $CWD/$ARCHIVE
	elif [[ $TYPE == "stable" ]]; then
		git archive --format=tar.gz --prefix="$ARCHIVE_DIR/" $(git describe --tags --match="v*" --abbrev=0) -o $CWD/$ARCHIVE
	fi

	echo "Start packaging..."
	echo " - \$TARGET  : $TARGET"
	echo " - \$TYPE    : $TYPE"
	echo " - \$GIT_ROOT: $GIT_ROOT"
	echo " - \$CWD     : $CWD"
	echo " - \$ARCHIVE : $ARCHIVE"

	# change directory and run
	cd $CWD
	package
	exit $?
else
	echo "Start deploying..."
	echo " - \$TARGET  : $TARGET"
	echo " - \$TYPE    : $TYPE"
	echo " - \$CWD     : $CWD"

	deploy
	exit $?
fi
